import unittest
from datetime import datetime
from unittest.mock import patch, Mock, MagicMock

from requests import Response

from youless_api import YoulessAPI

test_host = "192.1.1.1"


def mock_ls120_pvoutput(*args, **kwargs) -> Response:
    if args[0] == 'http://192.1.1.1/d':
        return Mock(
            ok=True,
            json=lambda: {'mac': '293:23fd:23'}
        )

    if args[0] == 'http://192.1.1.1/e':
        return Mock(
            ok=True,
            headers={'Content-Type': 'text/html'}
        )

    if args[0] == 'http://192.1.1.1/a?f=j':
        return Mock(
            ok=True,
            json=lambda: {
                "cnt": "141950,625",
                "pwr": 750,
                "lvl": 90,
                "dev": "(&plusmn;3%)",
                "det": "",
                "con": "OK",
                "sts": "(33)",
                "raw": 743
            })

    return Mock(ok=False)


def mock_ls120(*args, **kwargs) -> Response:
    if args[0] == 'http://192.1.1.1/d':
        return Mock(
            ok=True,
            json=lambda: {'mac': '293:23fd:23', 'fw': '1.6.0-EL'}
        )

    if args[0] == 'http://192.1.1.1/e':
        return Mock(
            ok=True,
            headers={'Content-Type': 'application/json'},
            json=lambda: [{
                "tm": 1611929119,
                "net": 9194.164,
                "pwr": 2382,
                "ts0": 1608654000,
                "cs0": 15.000,
                "ps0": 10,
                "p1": 4703.562,
                "p2": 4490.631,
                "n1": 0.029,
                "n2": 0.000,
                "gas": 1624.264,
                "gts": int(datetime.now().strftime("%y%m%d%H00")),
                "wtr": 1234.564,
                "wts": int(datetime.now().strftime("%y%m%d%H00"))
            }])

    if args[0] == 'http://192.1.1.1/f':
        return Mock(
            ok=True,
            json=lambda: {
                "tr": 1,
                "i1": 0.123,
                "v1": 240,
                "l1": 462,
                "v2": 240,
                "l2": 230,
                "i2": 0.123,
                "v3": 240,
                "l3": 230,
                "i3": 0.123,
                "pp": 1200,
                "pts": int(datetime.now().strftime("%y%m%d%H%M")),
                "pa": 400
            }
        )

    return Mock(ok=False)


def mock_ls120_reported(*args, **kwargs) -> Response:
    if args[0] == 'http://192.1.1.1/d':
        return Mock(
            ok=True,
            json=lambda: {'mac': '293:23fd:23', 'fw': '1.6.0-EL'}
        )

    if args[0] == 'http://192.1.1.1/e':
        return Mock(
            ok=True,
            headers={'Content-Type': 'application/json'},
            json=lambda: [{
                "tm": 1719966932,
                "net": 24277.256,
                "pwr": -6,
                "ts0": 1719964559,
                "cs0": 75.271,
                "ps0": 0,
                "p1": 13775.844,
                "p2": 12057.301,
                "n1": 439.157,
                "n2": 1116.732,
                "gas": 3754.789,
                "gts":int(datetime.now().strftime("%y%m%d%H00")),
                "wtr": 0.000,
                "wts": 0
            }])

    if args[0] == 'http://192.1.1.1/f':
        return Mock(
            ok=True,
            json=lambda: {
                "tr": 2,
                "pa": 0,
                "pp": 0,
                "pts": 0,
                "i1": 1.000,
                "i2": 2.000,
                "i3": 1.000,
                "v1": 233.900,
                "v2": 232.400,
                "v3": 233.600,
                "l1": 50,
                "l2": 199,
                "l3": -218}
        )

    return Mock(ok=False)


def mock_ls110_device(*args, **kwargs):
    if args[0] == 'http://192.1.1.1/d':
        return Mock(ok=False)
    if args[0] == 'http://192.1.1.1/a?f=j':
        return Mock(
            ok=True,
            json=lambda: {
                "cnt": "141950,625",
                "pwr": 750,
                "lvl": 90,
                "dev": "(&plusmn;3%)",
                "det": "",
                "con": "OK",
                "sts": "(33)",
                "raw": 743
            })

    return Mock(ok=False)


class YoulessAPITest(unittest.TestCase):

    @patch('youless_api.gateway.requests.get', side_effect=mock_ls120)
    def test_device_ls120(self, mock_get: MagicMock):
        api = YoulessAPI(test_host)
        api.initialize()
        api.update()

        self.assertEqual(api.model, 'LS120')
        self.assertEqual(api.mac_address, '293:23fd:23')
        self.assertEqual(api.firmware_version, '1.6.0-EL')
        self.assertEqual(api.current_tariff, 1)

        self.assertEqual(api.water_meter.value, 1234.564)
        self.assertEqual(api.extra_meter.total.value, 15.0)
        self.assertEqual(api.extra_meter.usage.value, 10)
        self.assertEqual(api.delivery_meter.low.value, 0.029)
        self.assertEqual(api.delivery_meter.high.value, 0.0)
        self.assertEqual(api.gas_meter.value, 1624.264)
        self.assertEqual(api.current_power_usage.value, 2382)
        self.assertEqual(api.power_meter.high.value, 4490.631)
        self.assertEqual(api.power_meter.low.value, 4703.562)
        self.assertEqual(9194.164, api.power_meter.total.value)
        self.assertEqual(400, api.average_power.value)
        self.assertEqual(1200, api.peak_power.value)
        self.assertEqual(datetime.now().replace(second=0, microsecond=0), api.peak_power_time)

        mock_get.assert_any_call('http://192.1.1.1/d', auth=None, timeout=2)
        mock_get.assert_any_call('http://192.1.1.1/e', auth=None, timeout=2)
        mock_get.assert_any_call('http://192.1.1.1/f', auth=None, timeout=2)

    @patch('youless_api.gateway.requests.get', side_effect=mock_ls120_reported)
    def test_device_ls120_reported_issues(self, mock_get: MagicMock):
        api = YoulessAPI(test_host)
        api.initialize()
        api.update()

        self.assertEqual(api.extra_meter.total.value, 75.271)
        self.assertEqual(api.extra_meter.usage.value, 0)

    @patch('youless_api.gateway.requests.get', side_effect=mock_ls120)
    def test_device_ls120_authenticated(self, mock_get: MagicMock):
        api = YoulessAPI(test_host, 'admin', 'password')
        api.initialize()

        self.assertEqual(api.model, 'LS120')
        mock_get.assert_any_call('http://192.1.1.1/d', auth=('admin', 'password'), timeout=2)
        mock_get.assert_any_call('http://192.1.1.1/e', auth=('admin', 'password'), timeout=2)

    @patch('youless_api.gateway.requests.get', side_effect=mock_ls120_pvoutput)
    def test_ls120_firmare_pvoutput(self, mock_get: MagicMock):
        api = YoulessAPI(test_host)
        api.initialize()
        api.update()

        self.assertEqual(api.model, 'LS120 - PVOutput')
        self.assertEqual(api.mac_address, '293:23fd:23')
        mock_get.assert_any_call('http://192.1.1.1/d', auth=None, timeout=2)
        mock_get.assert_any_call('http://192.1.1.1/e', auth=None, timeout=2)

    @patch('youless_api.gateway.requests.get', side_effect=mock_ls120_pvoutput)
    def test_ls120_firmare_pvoutput_authenticated(self, mock_get: MagicMock):
        api = YoulessAPI(test_host, 'admin', 'password')
        api.initialize()

        self.assertEqual(api.model, 'LS120 - PVOutput')
        self.assertEqual(api.mac_address, '293:23fd:23')
        mock_get.assert_any_call('http://192.1.1.1/d', auth=('admin', 'password'), timeout=2)
        mock_get.assert_any_call('http://192.1.1.1/e', auth=('admin', 'password'), timeout=2)

    @patch('youless_api.gateway.requests.get', side_effect=mock_ls110_device)
    def test_device_ls110(self, mock_get: MagicMock):
        api = YoulessAPI(test_host)
        api.initialize()

        mock_get.assert_called_with('http://192.1.1.1/d', auth=None, timeout=2)
        self.assertEqual(api.model, 'LS110')
        self.assertIsNone(api.mac_address)

        api.update()
        mock_get.assert_called_with('http://192.1.1.1/a?f=j', auth=None, timeout=2)

    @patch('youless_api.gateway.requests.get', side_effect=mock_ls110_device)
    def test_device_ls110_authenticated(self, mock_get: MagicMock):
        api = YoulessAPI(test_host, 'admin', 'password')
        api.initialize()
        mock_get.assert_called_with('http://192.1.1.1/d', auth=('admin', 'password'), timeout=2)

        self.assertEqual(api.model, 'LS110')
        self.assertIsNone(api.mac_address)



from datetime import datetime
from unittest import TestCase
from unittest.mock import patch, Mock, MagicMock

from requests import Response

from youless_api.gateway import fetch_enologic_api


def mock_ls120_ok(*args, **kwargs) -> Response:
    if args[0] == 'http://localhost/e':
        return Mock(
            ok=True,
            headers={'Content-Type': 'application/json'},
            json=lambda: [{
                "tm": 1611929119,
                "net": 9194.164,
                "pwr": 2382,
                "ts0": 1608654000,
                "cs0": 0.000,
                "ps0": 0,
                "p1": 4703.562,
                "p2": 4490.631,
                "n1": 0.029,
                "n2": 0.000,
                "gas": 1624.264,
                "gts": int(datetime.now().strftime("%y%m%d%H00")),
                "wtr": 1234.564,
                "wts": int(datetime.now().strftime("%y%m%d%H00"))
            }]
        )

    return Mock(ok=False)


def mock_stale_gas(*args, **kwargs) -> Response:
    if args[0] == 'http://localhost/e':
        return Mock(
            ok=True,
            headers={'Content-Type': 'application/json'},
            json=lambda: [{
                "tm": 1611929119,
                "net": 9194.164,
                "pwr": 2382,
                "ts0": 1608654000,
                "cs0": 15.000,
                "ps0": 10,
                "p1": 4703.562,
                "p2": 4490.631,
                "n1": 0.029,
                "n2": 0.000,
                "gas": 1624.264,
                "gts": 3894900,
                "wtr": 1234.564,
                "wts": 3894900
            }]
        )

    return Mock(ok=False)


def mock_enologic_missing_values(*args, **kwargs) -> Response:
    if args[0] == 'http://localhost/e':
        return Mock(
            ok=True,
            headers={'Content-Type': 'application/json'},
            json=lambda: [{
                "tm": 1611929119,
                "net": 9194.164,
                "pwr": 2382,
                "ts0": 1608654000,
                "cs0": 0.000,
                "ps0": 0,
                "gas": 1624.264,
                "gts": int(datetime.now().strftime("%y%m%d%H00")),
                "wtr": 1234.564,
                "wts": int(datetime.now().strftime("%y%m%d%H00"))
            }]
        )

    return Mock(ok=False)


def mock_enologic_error(*args, **kwargs) -> Response:
    return Mock(ok=False)


class GatewayTest(TestCase):

    @patch('youless_api.gateway.requests.get', side_effect=mock_ls120_ok)
    def test_enologic_correct(self, mock_get: MagicMock):
        dataset = fetch_enologic_api('localhost', None)

        self.assertEqual(dataset['net'], 9194.164)
        self.assertEqual(dataset['p2'], 4490.631)
        self.assertEqual(dataset['p1'], 4703.562)
        self.assertEqual(dataset['pwr'], 2382)
        self.assertEqual(dataset['gas'], 1624.264)
        self.assertEqual(dataset['wtr'], 1234.564)
        self.assertEqual(dataset['cs0'], 0.000)
        self.assertEqual(dataset['n1'], 0.029)
        mock_get.assert_any_call('http://localhost/e', auth=None, timeout=2)

    @patch('youless_api.gateway.requests.get', side_effect=mock_stale_gas)
    def test_enologic_stale_gas(self, mock_get: MagicMock):
        dataset = fetch_enologic_api('localhost', None)

        self.assertEqual(dataset['net'], 9194.164)
        self.assertEqual(dataset['p2'], 4490.631)
        self.assertEqual(dataset['p1'], 4703.562)
        self.assertEqual(dataset['pwr'], 2382)
        self.assertEqual(dataset['gas'], None)
        self.assertEqual(dataset['wtr'], None)
        self.assertEqual(dataset['cs0'], 15.000)
        self.assertEqual(dataset['ps0'], 10)
        self.assertEqual(dataset['n1'], 0.029)
        mock_get.assert_any_call('http://localhost/e', auth=None, timeout=2)

    @patch('youless_api.gateway.requests.get', side_effect=mock_enologic_missing_values)
    def test_enologic_missing_p_and_n(self, mock_get: MagicMock):
        dataset = fetch_enologic_api('localhost', None)

        self.assertEqual(dataset['p1'], None)
        self.assertEqual(dataset['p2'], None)
        self.assertEqual(dataset['n1'], None)
        self.assertEqual(dataset['n2'], None)
        mock_get.assert_any_call('http://localhost/e', auth=None, timeout=2)

    @patch('youless_api.gateway.requests.get', side_effect=mock_enologic_error)
    def test_enologic_error(self, mock_get: MagicMock):
        dataset = fetch_enologic_api('localhost', None)

        self.assertEqual(dataset, None)
        mock_get.assert_any_call('http://localhost/e', auth=None, timeout=2)




"""
This file contains a helper class to easily obtain data from the YouLess sensor.
"""
import logging
from datetime import datetime
from typing import Optional

from youless_api.const import SensorType
from youless_api.device import ls110, ls120
from youless_api.gateway import fetch_device_info, fetch_enologic_api
from youless_api.youless_sensor import YoulessSensor, PowerMeter, ExtraMeter, DeliveryMeter, Phase

name = "youless_api"
logger = logging.getLogger(__name__)


class YoulessAPI:
    """A helper class to obtain data from the YouLess Sensor."""

    _cache_data: Optional[dict] = None

    def __init__(self, host, username=None, password=None):
        """Initialize the data bridge."""
        self._host = host
        if username is None:
            self._authentication = None
        else:
            self._authentication = (username, password)
        self._model = None
        self._mac_address = None
        self._firmware_version = None
        self._fetcher = None

    def initialize(self):
        """Establish a connection to the remote device"""
        device_info = fetch_device_info(self._host, self._authentication)

        if device_info is None:
            logger.debug("No device information discovered, assuming LS110 device.")
            self._model = "LS110"
            self._fetcher = ls110(self._host, self._authentication)
        else:
            self._mac_address = device_info["mac"]
            self._firmware_version = device_info["fw"] if "fw" in device_info else None

            enologic_data = fetch_enologic_api(self._host, self._authentication)
            if enologic_data is None:
                logger.debug("Incorrect enologic response, assuming LS120 with PVOutput firmware.")
                self._model = "LS120 - PVOutput"
                self._fetcher = ls110(self._host, self._authentication)
            else:
                logger.debug("Enologic output detected, LS120 device found.")
                self._model = "LS120"
                self._fetcher = ls120(self._host, self._authentication, device_info)

    def update(self):
        """Fetch the latest settings from the Youless Sensor."""
        if self._fetcher:
            self._cache_data = self._fetcher()
        else:
            logger.warning("No fetch algorithm is chosen, setup failed.")

    @property
    def mac_address(self) -> Optional[str]:
        """Get the MAC address of the connected device."""
        return self._mac_address

    @property
    def model(self) -> Optional[str]:
        """Return the model of the connected device."""
        return self._model

    @property
    def firmware_version(self) -> Optional[str]:
        """Get the firmware version of the connected device."""
        return self._firmware_version

    @property
    def current_tariff(self) -> Optional[int]:
        """Get the current tariff, is either 0 or 1 and only present if phase information is present."""
        return self._cache_data[SensorType.TARIFF] if SensorType.TARIFF in self._cache_data else None

    @property
    def water_meter(self) -> Optional[YoulessSensor]:
        """Get the water data available."""
        return self._cache_data[SensorType.WATER] if SensorType.WATER in self._cache_data else None

    @property
    def gas_meter(self) -> Optional[YoulessSensor]:
        """"Get the gas data available."""
        return self._cache_data[SensorType.GAS] if SensorType.GAS in self._cache_data else None

    @property
    def current_power_usage(self) -> Optional[YoulessSensor]:
        """Get the current power usage."""
        return self._cache_data[SensorType.POWER_USAGE] if SensorType.POWER_USAGE in self._cache_data else None

    @property
    def average_power(self) -> Optional[YoulessSensor]:
        """Get the average power usage of active Tarif."""
        return self._cache_data[SensorType.POWER_AVERAGE] if SensorType.POWER_AVERAGE in self._cache_data else None

    @property
    def power_meter(self) -> Optional[PowerMeter]:
        """Get the power meter values."""
        return self._cache_data[SensorType.POWER_METER] if SensorType.POWER_METER in self._cache_data else None

    @property
    def delivery_meter(self) -> Optional[DeliveryMeter]:
        """Get the power delivered values."""
        return self._cache_data[SensorType.DELIVERY_METER] if SensorType.DELIVERY_METER in self._cache_data else None

    @property
    def extra_meter(self) -> Optional[ExtraMeter]:
        """Get the meter values of an attached meter."""
        return self._cache_data[SensorType.EXTRA_METER] if SensorType.EXTRA_METER in self._cache_data else None

    @property
    def phase1(self) -> Optional[Phase]:
        """Get the phase 1 information"""
        return self._cache_data[SensorType.PHASE1] if SensorType.PHASE1 in self._cache_data else None

    @property
    def phase2(self) -> Optional[Phase]:
        """Get the phase 1 information"""
        return self._cache_data[SensorType.PHASE2] if SensorType.PHASE2 in self._cache_data else None

    @property
    def phase3(self) -> Optional[Phase]:
        """Get the phase 1 information"""
        return self._cache_data[SensorType.PHASE3] if SensorType.PHASE3 in self._cache_data else None

    @property
    def peak_power(self) -> Optional[YoulessSensor]:
        """Get the peak power of the month."""
        return self._cache_data[SensorType.MONTH_PEAK] if SensorType.MONTH_PEAK in self._cache_data else None

    @property
    def peak_power_time(self) -> Optional[datetime]:
        """Get the date time of the peak time."""
        return self._cache_data[SensorType.MONTH_PEAK_TIME] if SensorType.MONTH_PEAK_TIME in self._cache_data else None

    @property
    def secured(self) -> bool:
        """Flag indicating if the API has authentication or not."""
        return self._authentication is not None



import unittest

from youless_api.youless_sensor import YoulessSensor


class TestYoulessData(unittest.TestCase):

    def test_value(self):
        value = YoulessSensor(1.232, "w")
        self.assertTrue(value.value == 1.232)
        self.assertTrue(value.unit_of_measurement == "w")



from datetime import datetime
from typing import Optional

import requests


def fetch_generic_api(host, authentication=None) -> Optional[dict]:
    """Fetches the data from the Youless API on the /a endpoint."""
    response = requests.get(f"http://{host}/a?f=j", auth=authentication, timeout=2)

    if response.ok:
        corrected = {**{'cs0': None, 'ps0': None}, **response.json()}
        parse_float_values_for = ['cnt', 'cs0']

        for correct_value in parse_float_values_for:
            if correct_value in corrected and corrected[correct_value] is not None:
                corrected[correct_value] = float(corrected[correct_value].replace(",", "."))

        return corrected

    return None


def fetch_phase_api(host, authentication=None) -> Optional[dict]:
    """Fetches the data from the Youless API on the /f endpoint."""
    response = requests.get(f"http://{host}/f", auth=authentication, timeout=2)

    return response.json() if response.ok else {}


def fetch_enologic_api(host, authentication=None):
    """Fetches the data from the Youless API on the /e endpoint."""
    response = requests.get(f"http://{host}/e", auth=authentication, timeout=2)

    if response.ok and response.headers['Content-Type'] == 'application/json':
        """Use fallback values if specific sensor values are missing."""
        corrected = {
            **{
                'p1': None,
                'p2': None,
                'n1': None,
                'n2': None,
                'gas': None,
                'wtr': None,
                'pwr': None,
                'cs0': None,
                'ps0': None},
            **response.json()[0]
        }
        if 'gts' in corrected:
            formatted_date = datetime.now().strftime("%y%m%d") + "0000"
            if corrected["gts"] != 0 and int(formatted_date) >= corrected["gts"]:
                corrected["gas"] = None

        if 'wts' in corrected:
            formatted_date = datetime.now().strftime("%y%m%d") + "0000"
            if corrected["wts"] != 0 and int(formatted_date) >= corrected["wts"]:
                corrected["wtr"] = None

        return corrected

    return None


def fetch_device_info(host, authentication=None) -> Optional[dict]:
    """Fetch the device information from the Youless device."""
    response = requests.get(f"http://{host}/d", auth=authentication, timeout=2)

    return response.json() if response.ok else None




URI_ELOGIC = '/e'
URI_DEVICE_INFO = '/d'
URI_GENERIC = '/a?f=j'
URI_PHASES = '/f'


class MockResponse:

    def __init__(self):
        self._ok = False
        self._json = lambda: 0
        self._headers = {}
        self._text = ''

    def setup(self, ok, json, text, headers):
        self._ok = ok
        self._json = json
        self._text = text
        self._headers = headers

    @property
    def ok(self):
        return self._ok

    def json(self):
        return self._json()

    @property
    def headers(self):
        return self._headers



from datetime import datetime
from typing import Optional

from youless_api.gateway import fetch_enologic_api, fetch_generic_api, fetch_phase_api
from youless_api.youless_sensor import YoulessSensor, PowerMeter, DeliveryMeter, ExtraMeter, Phase
from youless_api.const import SensorType


def ls110(host, authentication):
    """The device wrapper for the LS110, will return a function to fetch the latest information."""
    def update() -> Optional[dict]:
        """The actual method to refresh the data."""
        dataset = fetch_generic_api(host, authentication)
        if dataset is None:
            return None

        return {
            SensorType.POWER_METER: PowerMeter(
                YoulessSensor(None, None),
                YoulessSensor(None, None),
                YoulessSensor(dataset['cnt'], 'kWh')
            ),
            SensorType.POWER_USAGE: YoulessSensor(dataset['pwr'], 'W'),
            SensorType.EXTRA_METER: ExtraMeter(
                YoulessSensor(dataset['cs0'], 'kWh'),
                YoulessSensor(dataset['ps0'], 'W')
            )
        }

    return update


def ls120(host, authentication, device_info):
    """The device wrapper for the LS120, will return a function to fetch the latest information."""
    supports_phases = False
    if 'fw' in device_info:
        supports_phases = float(device_info['fw'][0:3]) >= 1.5

    def update() -> Optional[dict]:
        """The actual method to refresh the data."""
        dataset = fetch_enologic_api(host, authentication)
        phase_info = fetch_phase_api(host, authentication) if supports_phases else {}

        if dataset is None:
            return None

        return {
            SensorType.GAS: YoulessSensor(dataset['gas'], 'm3'),
            SensorType.WATER: YoulessSensor(dataset['wtr'], 'm3'),
            SensorType.POWER_USAGE: YoulessSensor(dataset['pwr'], 'W'),
            SensorType.POWER_METER: PowerMeter(
                YoulessSensor(dataset['p1'], 'kWh'),
                YoulessSensor(dataset['p2'], 'kWh'),
                YoulessSensor(dataset['net'], 'kWh')
            ),
            SensorType.DELIVERY_METER: DeliveryMeter(
                YoulessSensor(dataset['n1'], 'kWh'),
                YoulessSensor(dataset['n2'], 'kWh')
            ),
            SensorType.EXTRA_METER: ExtraMeter(
                YoulessSensor(dataset['cs0'], 'kWh'),
                YoulessSensor(dataset['ps0'], 'W')
            ),
            SensorType.PHASE1: Phase(
                YoulessSensor(phase_info['i1'], 'A'),
                YoulessSensor(phase_info['v1'], 'V'),
                YoulessSensor(phase_info['l1'], 'W')) if 'i1' in phase_info else None,
            SensorType.PHASE2: Phase(
                YoulessSensor(phase_info['i2'], 'A'),
                YoulessSensor(phase_info['v2'], 'V'),
                YoulessSensor(phase_info['l2'], '')) if 'i2' in phase_info else None,
            SensorType.PHASE3: Phase(
                YoulessSensor(phase_info['i3'], 'A'),
                YoulessSensor(phase_info['v3'], 'V'),
                YoulessSensor(phase_info['l3'], 'W')) if 'i3' in phase_info else None,
            SensorType.TARIFF: phase_info['tr'] if phase_info else None,
            SensorType.MONTH_PEAK: YoulessSensor(phase_info['pp'], 'W') if 'pp' in phase_info else None,
            SensorType.POWER_AVERAGE: YoulessSensor(phase_info['pa'], 'W') if 'pa' in phase_info else None,
            SensorType.MONTH_PEAK_TIME: datetime.strptime(str(phase_info['pts']), '%y%m%d%H%M') if 'pts' in phase_info and phase_info['pts'] > 0 else None,
        }

    return update




from unittest import TestCase
from unittest.mock import patch, Mock, MagicMock

from requests import Response

from youless_api.gateway import fetch_generic_api


def mock_generic_ok(*args, **kwargs) -> Response:
    return Mock(
        ok=True,
        json=lambda: {
                "cnt": "141950,625",
                "pwr": 750,
                "lvl": 90,
                "dev": "(&plusmn;3%)",
                "det": "",
                "con": "OK",
                "sts": "(33)",
                "raw": 743
            })


class GatewayGenericTest(TestCase):

    @patch('youless_api.gateway.requests.get', side_effect=mock_generic_ok)
    def test_generic_ok(self, mock_get: MagicMock):
        dataset = fetch_generic_api('localhost', None)

        self.assertEqual(dataset['cnt'], 141950.625)
        self.assertEqual(dataset['pwr'], 750)
        self.assertEqual(dataset['lvl'], 90)
        self.assertEqual(dataset['dev'], '(&plusmn;3%)')
        self.assertEqual(dataset['det'], "")
        self.assertEqual(dataset['con'], 'OK')
        self.assertEqual(dataset['sts'], "(33)")
        self.assertEqual(dataset['raw'], 743)

        mock_get.assert_any_call('http://localhost/a?f=j', auth=None, timeout=2)



import os
import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

default_version = '2.0.0'
version = os.getenv('PUBLISH_VERSION', default_version)

setuptools.setup(
      name='youless_api',
      version=version,
      description='A bridge for python to the YouLess sensor',
      long_description=long_description,
      long_description_content_type="text/markdown",
      url='https://github.com/gjong/youless-python-bridge',
      author='G. Jongerius',
      license='MIT',
      packages=setuptools.find_packages(exclude=("test",)),
      zip_safe=False)



"""

This file contains the sensor class for the youless API

"""


class YoulessSensor:
    """A wrapper class to contain the Youless Sensor values."""

    def __init__(self, value, uom):
        """Initialize the value wrapper."""
        self._value = value
        self._uom = uom

    @property
    def unit_of_measurement(self):
        """Get the unit of measurement for this value."""
        return self._uom

    @property
    def value(self):
        """Get the current value"""
        return self._value


class PowerMeter:

    def __init__(self, low: YoulessSensor, high: YoulessSensor,
                 total: YoulessSensor):
        self._low = low
        self._high = high
        self._total = total

    @property
    def low(self) -> YoulessSensor:
        return self._low

    @property
    def high(self) -> YoulessSensor:
        return self._high

    @property
    def total(self):
        return self._total


class DeliveryMeter:

    def __init__(self, low: YoulessSensor, high: YoulessSensor):
        self._low = low
        self._high = high

    @property
    def low(self) -> YoulessSensor:
        return self._low

    @property
    def high(self) -> YoulessSensor:
        return self._high


class ExtraMeter:

    def __init__(self, total: YoulessSensor, usage: YoulessSensor):
        self._total = total
        self._usage = usage

    @property
    def usage(self) -> YoulessSensor:
        return self._usage

    @property
    def total(self) -> YoulessSensor:
        return self._total


class Phase:
    def __init__(self, current: YoulessSensor, voltage: YoulessSensor, power: YoulessSensor):
        self._current = current
        self._voltage = voltage
        self._power = power

    @property
    def current(self) -> YoulessSensor:
        return self._current

    @property
    def voltage(self) -> YoulessSensor:
        return self._voltage

    @property
    def power(self) -> YoulessSensor:
        return self._power



from enum import Enum

STATE_OK = "ok"
STATE_FAILED = "failed"


class SensorType(Enum):
    """The sensor type class creates an enumeration of the supported sensors by YouLess."""
    WATER = "water"
    GAS = "gas"
    POWER_USAGE = "power_usage"
    POWER_METER = "power_meter"
    DELIVERY_METER = "delivery_meter"
    EXTRA_METER = "extra_meter"
    PHASE1 = "phase1"
    PHASE2 = "phase2"
    PHASE3 = "phase3"
    TARIFF = "tariff"
    MONTH_PEAK = "month_peak"
    MONTH_PEAK_TIME = "month_peak_time"
    POWER_AVERAGE = "power_average"
